--/ Example 13

--/ Naturally we can combine the use of JOINS with WHERE, GROUP BY clauses and AGGREGATE functions to further
--/ filter the data returned in a result set

SELECT 
	o.orderID, o.CustomerID, CONVERT(VARCHAR(10), o.OrderDate, 103) AS OrderDateAsString,
	od.Quantity, od.UnitPrice, 
	p.ProductName,
	c.CompanyName AS CustomerCompanyName,
	s.CompanyName AS SupplierCompanyName
FROM 	Orders o 
	JOIN [Order Details] od ON o.orderID = od.orderID 
	JOIN Products p ON od.ProductID = p.ProductID
	JOIN Customers c ON c.CustomerID = o.CustomerID
	JOIN Suppliers s ON p.supplierID = s.supplierID
WHERE	s.Country LIKE '%Germany%'
	AND p.Discontinued = 0
	AND od.Discount <> 0.0


--/ and in another we can see what value discontinued products have been sold grouped by their suppliers country of origin
SELECT 
	s.Country, SUM(od.Quantity * od.UnitPrice) AS OrderValue 
FROM 	Orders o 
	JOIN [Order Details] od ON o.orderID = od.orderID 
	JOIN Products p ON od.ProductID = p.ProductID
	JOIN Customers c ON c.CustomerID = o.CustomerID
	JOIN Suppliers s ON p.supplierID = s.supplierID
WHERE   p.Discontinued = 1
GROUP BY s.Country
ORDER BY OrderValue

--/ no doubt the next question would be, well what are those discountinued products?, you need to add the product code 
--/ and details to the query

SELECT 
	s.Country, p.ProductName, p.ProductID, SUM(od.Quantity * od.UnitPrice) AS OrderValue 
FROM 	Orders o 
	JOIN [Order Details] od ON o.orderID = od.orderID 
	JOIN Products p ON od.ProductID = p.ProductID
	JOIN Customers c ON c.CustomerID = o.CustomerID
	JOIN Suppliers s ON p.supplierID = s.supplierID
WHERE   p.Discontinued = 1
GROUP BY s.Country, p.ProductName, p.ProductID
ORDER BY OrderValue

--/ However as the query grows in complexity, adding columns/fields when solving questions like 'which employees' sold
--/ these products, you end up hitting the problem with the limiting GROUP BY clause. When this occurs, it may be time
--/ to treat this query as a 'table/view' in its own right and use a table alias to join it back on other tables so...

SELECT  e.firstname + ' ' + e.lastname AS EmployeeName,
	SUM(od.Quantity * od.UnitPrice) AS OrderValue
	FROM Employees e
	JOIN Orders o ON o.EmployeeID = e.EmployeeID
	JOIN [Order Details] od ON od.orderID = o.orderID
	JOIN products p ON od.ProductID = p.ProductID
	JOIN
	--/now join to the previous query using a table alias for it called 'q'
(
SELECT 
	s.Country, p.ProductName, p.ProductID, SUM(od.Quantity * od.UnitPrice) AS OrderValue 
FROM 	Orders o 
	JOIN [Order Details] od ON o.orderID = od.orderID 
	JOIN Products p ON od.ProductID = p.ProductID
	JOIN Customers c ON c.CustomerID = o.CustomerID
	JOIN Suppliers s ON p.supplierID = s.supplierID
WHERE   p.Discontinued = 1
GROUP BY s.Country, p.ProductName, p.ProductID
) q	ON q.ProductName = p.ProductName

GROUP BY e.firstname + ' ' + e.lastname
ORDER BY OrderValue DESC








